Package: SynergyLMM
Title: Statistical Framework for in Vivo Drug Combination Studies
Version: 1.1.3
Authors@R: 
    c(person("Rafael", "Romero-Becerra", , "rafrombec@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-3935-2647")), 
           person("Zhi", "Zhao", email = "zhi.zhao@medisin.uio.no", role = c("ctb")), 
           person("Tero", "Aittokallio", email = "t.a.aittokallio@medisin.uio.no", role = c("ctb")))
Description: A framework for evaluating drug combination effects in preclinical in vivo studies.
    'SynergyLMM' provides functions to analyze longitudinal tumor growth experiments using mixed-effects models, 
    perform time-resolved analyses of synergy and antagonism, evaluate model diagnostics and performance, 
    and assess both post-hoc and a priori statistical power.
    The calculation of drug combination synergy follows the statistical framework provided by Demidenko and Miller (2019, <doi:10.1371/journal.pone.0224137>).
    The implementation and analysis of linear mixed-effect models is based on the methods described by Pinheiro and Bates (2000, <doi:10.1007/b98882>),
    and Gałecki and Burzykowski (2013, <doi:10.1007/978-1-4614-3900-4>).
License: GPL (>= 3)
Encoding: UTF-8
Imports: magrittr (>= 2.0.3), rlang, dplyr (>= 1.1.4), ggplot2 (>=
        3.5.1), cowplot (>= 1.1.3), nlme (>= 3.1-165), nlmeU, fBasics,
        car, MASS, performance, lattice, marginaleffects, clubSandwich
Suggests: lme4, knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
Depends: R (>= 4.0)
LazyData: true
VignetteBuilder: knitr
URL: https://github.com/RafRomB/SynergyLMM
BugReports: https://github.com/RafRomB/SynergyLMM/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-11 21:00:11 UTC; rafrombec
Author: Rafael Romero-Becerra [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3935-2647>),
  Zhi Zhao [ctb],
  Tero Aittokallio [ctb]
Maintainer: Rafael Romero-Becerra <rafrombec@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-12 18:40:48 UTC
Built: R 4.5.2; ; 2026-05-12 19:33:29 UTC; unix
