# Generated by roxygen2: do not edit by hand

S3method(Pwr,lme)
S3method(lmmModel_estimates,explme)
S3method(lmmModel_estimates,gompertzlme)
S3method(lmmSynergy,explme)
S3method(lmmSynergy,gompertzlme)
export("%>%")
export(APrioriPwr)
export(CookDistance)
export(ObsvsPred)
export(PostHocPwr)
export(Pwr)
export(PwrSampleSize)
export(PwrTime)
export(getRTV)
export(lmmModel)
export(lmmModel_estimates)
export(lmmSynergy)
export(logLikSubjectDisplacements)
export(plot_ObsvsPred)
export(plot_SynergyLMM)
export(plot_lmmModel)
export(plot_lmmSynergy)
export(plot_ranefDiagnostics)
export(plot_residDiagnostics)
export(ranefDiagnostics)
export(residDiagnostics)
export(simulateTumorGrowth)
import(graphics)
import(lattice)
import(nlme)
import(stats)
importFrom(cowplot,plot_grid)
importFrom(cowplot,theme_cowplot)
importFrom(fBasics,normalTest)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,arrow)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_continuous)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,stat_qq)
importFrom(ggplot2,stat_qq_line)
importFrom(ggplot2,theme)
importFrom(ggplot2,unit)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(magrittr,"%>%")
importFrom(marginaleffects,hypotheses)
importFrom(nlme,lme)
importFrom(nlme,lmeControl)
importFrom(nlme,pdDiag)
importFrom(rlang,.data)
importFrom(stats,fitted)
importFrom(stats,na.omit)
importFrom(stats,qqnorm)
importFrom(stats,resid)
importFrom(stats,residuals)
importFrom(utils,getFromNamespace)
